{% extends "base.html" %}
{% load static %}
{% block body %}

<div class="card bg-light">
  <div class="card-header">Add subnet to {{group.name}} group</div>
  <div class="card-body">
    <p class="card-text">
    <form action="{% url "add_group_subnet" group_id=group_id%}" method="post">
      {% csrf_token %}
      <table>
      {{form.as_table}}
      </table>
      <input type="submit" value="Submit"/>
    </form>
    </p>
  </div>
</div>
{% endblock %}
