IPTABLES_SAVE = [
    "# Generated by iptables-save v1.6.0 on Fri May 10 10:17:18 2019",
    "*filter",
    ":INPUT ACCEPT [27:6332]",
    ":FORWARD ACCEPT [0:0]",
    ":OUTPUT ACCEPT [1:40]",
    "COMMIT",
    "# Completed on Fri May 10 10:17:18 2019\n"
]
