{% extends "base.html" %}
{% load static %}
{% block body %}

<div class="card bg-light">
  <div class="card-header">Group membership for {{name}}</div>
  <div class="card-body">
    <p class="card-text">
    <form action="{% url "user_groups" user_id=user_id %}" method="post">
      {% csrf_token %}
      <table>
      {{form.as_table}}
      </table>
      <input type="submit" value="Submit"/>
    </form>
    </p>
  </div>
</div>
{% endblock %}
